// ES6 Component
// Import React and ReactDOM
import React from 'react';
import ReactDOM from 'react-dom';
var fs = require("fs");
var path = require('path')
const electron = window.require("electron")
const userDataPath = (electron.app || electron.remote.app).getPath('userData');
const dialog = (electron.dialog || electron.remote.dialog)
var moment = require('moment');
var blobUtil = require('blob-util')

require("font-awesome-webpack");

import {Tab, Tabs} from './ui';
import {InputToolbar, QsoListToolbar} from './toolbar';
import InputBox from './inputbox';
import StatsBox from './statsbox';
import SettingsBox from './settingsBox'
import LogBox from './logbox';
import FooterBox from './footer';

import helpers from './helpers'
import { logbook } from './logbook'
import dxcc from './dxcc';
import { eqsl } from './eqsl';
import { wsjtx } from './wsjtx'
import { fllog } from './fllog'

String.prototype.replaceAll = function(search, replacement) {
    var target = this;
    return target.replace(new RegExp(search, 'g'), replacement);
};

//------------------------------------------------------------------------------
// Search component created as a class
class App extends React.Component {
    constructor(props) {
      super(props);
      try {
        this.config = JSON.parse(fs.readFileSync(path.join(userDataPath, './config.json')));
      }
      catch (x) {
        this.config = {
            "callsign" : "NO0CALL",
            "grid" : "",
            "rig" : [],
            "antenna" : [],
            "hamqth" : { "username":"callsign", "password":"password" },
            "eqsl" : { "username":"callsign", "password":"password", "msg":"tnx for qso. 73 GL" },
            "pageSize" : "Letter"
        }
        fs.writeFile(path.join(userDataPath, './config.json'), JSON.stringify(this.config, null, 2) , 'utf8', ()=>{} ); // write it back
      }
      this.state = { searchText:null, freq:"14", mode:"SSB", logbook:[], isLoaded:false, lastSeen:null, pluginStat:{}}

      this.logbook = new logbook();
      this.logbook.setDxcc (dxcc);
      this.logbook.on('change', (logbook) => { this.setState({logbook:logbook, isLoaded:true}) });
      this.logbook.on('status', (obj) => { this.setState({pluginStat: Object.assign({}, this.state.pluginStat, obj)}) })
      this.logbook.syncWith("uberlog", "uberlog", "http://ido.dvrdns.org:5984/", "log_"+this.config.callsign.toLowerCase())
      //this.logbook.loadFromFile(path.join(userDataPath,"uberlog.adif"))

      dxcc.load().then(()=>{this.logbook.updateLogbook()});

      this.eqsl = new eqsl(this.config.eqsl.username, this.config.eqsl.password);
      this.eqsl.on('status', (obj) => { this.setState({pluginStat: Object.assign({}, this.state.pluginStat, obj)}) })
      //if (!fs.existsSync(path.join(userDataPath,"QSLs")))
      //  fs.mkdirSync(path.join(userDataPath,"QSLs"));

      this.wsjtx = new wsjtx();
      this.wsjtx.on('qso', this.handleWsjtxQso.bind(this))
      this.wsjtx.on('heartbeat', this.heartbeat.bind(this))
      this.wsjtx.on('status', (obj) => { this.setState({pluginStat: Object.assign({}, this.state.pluginStat, obj)}) })

      this.fllog = new fllog();
      this.fllog.on('status', (obj) => { this.setState({pluginStat: Object.assign({}, this.state.pluginStat, obj)}) })
      this.fllog.on('qso', this.handleFldigiQso.bind(this))
//      this.fllog.on("callsign", (qso)=>{console.log("FLDIGI",qso);this.handleNewCallsign(qso["CALL"])})
    }

    componentDidMount(){

    }



//------------------------------------------------------------------------------
//                            Toolbar
//------------------------------------------------------------------------------
    handleSearch(text){
       this.setState({searchText:text})
     }

    handleToolbarChanged(event){
      var newState = {}
      newState[event.target.id] = event.target.value;
      console.log(newState);
      this.setState(newState);
    }

    handleExportPDF(event){
        var file_path = dialog.showSaveDialog({
          properties: ['openFile'],
          filters: [
            {name: 'PDF', extensions: ['pdf']},
            {name: 'All Files', extensions: ['*']}
          ]
        })
        if (!file_path)
          return
        var print_win = new electron.remote.BrowserWindow({'auto-hide-menu-bar':true});
        var lines = []
        var pagenum = 0
        var theader = `<tr>
                      <td width="10px"></td>
                      <th width="80px">Date</th>
                      <th width="50px">Time</th>
                      <th width="90px">Callsign</th>
                      <th width="60px">Country</th>
                      <th colspan="2">Signal<br/>H | M</th>
                      <th width="40px">Freq</th>
                      <th width="40px">Mode</th>
                      <th width="20px">Name</th>
                      <th width="60px">QTH</th>
                      <th width="20px"></th>
                      <th colspan="2">QSL<br/>S|R</th>
                      <th>Remarks</th>
                      </tr>`
        var style =`<style>
                    #customers {
                        border-collapse: collapse;
                    }

                    #customers td, #customers th {
                        font-family: "Trebuchet MS", Arial, Helvetica, sans-serif;
                        vertical-align: bottom;
                        border: 0px solid #ddd;
                        padding: 2px;

                        overflow: hidden;
                        text-overflow: ellipsis;
                        white-space: nowrap;
                    }

                    #customers tr:nth-child(even){
                      background-color: #f2f2f2;
                    }

                    #customers td.number {background-color: #ffffff;}

                    #customers th {
                        padding-top: 8px;
                        padding-bottom: 8px;
                        text-align: center;
                        background-color: #4CAF50;
                        color: white;
                    }
                    div.footnotes {
                      text-align: right;
                      page-break-after: always;
                    }
                    `
        lines.push('data:text/html;charset=UTF-8,')
        lines.push('<HTML><HEAD>')
        lines.push('<style>'+style+'</style>')
        lines.push('</HEAD><BODY>')
        lines.push('<table id="customers" style="width:100%">')
        lines.push(theader)
        var list = this.state.logbook
        list = list.sort((a,b)=>{return a["_id"]>b["_id"]?1:-1})
        list.map((qso, i)=>{
            var datetime = moment.utc(qso["QSO_DATE"] + " " + qso["TIME_ON"] , "YYYYMMDD HHmm").locale("en-gb")
            var remarks = [];
            if (qso["COMMENT"]) remarks.push(qso["COMMENT"])
            if (qso["MY_RIG"]) remarks.push(qso["MY_RIG"])
            if (qso["MY_ANTENNA"]) remarks.push(qso["MY_ANTENNA"])
            if (qso["MY_CITY"]) remarks.push(qso["MY_CITY"])
            lines.push("<tr>")
            lines.push('<td width="30px" class="number">'+(i+1)+"</td>")
            lines.push("<td>"+datetime.format("L")+"</td>")
            lines.push("<td>"+datetime.format("LT")+"</td>")
            lines.push("<td>"+qso["CALL"]+"</td>")
            lines.push("<td>"+qso["country_"]+"</td>")
            lines.push('<td width="40px">'+(qso["RST_SENT"]?qso["RST_SENT"]:"")+"</td>")
            lines.push('<td width="40px">'+(qso["RST_RCVD"]?qso["RST_RCVD"]:"")+"</td>")
            lines.push("<td>"+Math.floor(qso["FREQ"])+"</td>")
            lines.push("<td>"+qso["MODE"]+"</td>")
            lines.push("<td>"+(qso["NAME"]?qso["NAME"]:"")+"</td>")
            lines.push("<td>"+(qso["QTH"]?qso["QTH"]:"")+"</td>")
            lines.push('<td width="45px">'+(qso["GRID"]?qso["GRID"]:"")+"</td>")
            lines.push('<td width="10px">'+(qso["QSL_SENT"]=="Y"?"&#x2713;":"")+"</td>")
            lines.push('<td width="10px">'+(qso["QSL_RCVD"]=="Y"?"&#x2713;":"")+"</td>")
            lines.push("<td>"+remarks.join(" | ")+"</td>")
            lines.push("</tr>")
            if ((i+1)%25==0){
              pagenum += 1;
              lines.push("</table>")
              lines.push("<br/>")
              lines.push('<div class="footnotes">Page '+pagenum+'</div>')
              lines.push("<br/>")
              lines.push('<table id="customers" style="width:100%">')
              lines.push(theader)
            }

        });
        lines.push("</table>")
        lines.push("</BODY></HTML>")
        print_win.loadURL(lines.join('\n'));
        print_win.show();

          var option = {
            landscape: true,
            marginsType: 2,
            printBackground: true,
            printSelectionOnly: false,
            pageSize: this.config.pageSize, // was 'A4',
          };
        print_win.webContents.on('did-finish-load', () => {
          print_win.webContents.printToPDF(option, (err, data) => {
                 if (err) {
                   dialog.showErrorBox('Error', err);
                   return;
                 }
                 fs.writeFile(file_path, data, (err) => {
                   if (err) {
                     dialog.showErrorBox('Error', err);
                     return;
                   }
                   print_win.close()
                 });
               });
          });
        print_win.on('closed', function() {
          print_win = null;
        });
    }

    handleExportAdif(event){
      var fname = dialog.showSaveDialog({
        properties: ['openFile'],
        filters: [
          {name: 'ADIF', extensions: ['adi', 'adif']},
          {name: 'Text', extensions: ['txt', 'text']},
          {name: 'All Files', extensions: ['*']}
        ]
      })
      if (fname)
        this.logbook.writeLogbookToFile(fname)
    }

    handleImportAdif(event){
      var fname = dialog.showOpenDialog({
        properties: ['openFile'],
        filters: [
          {name: 'ADIF', extensions: ['adif']},
          {name: 'Text', extensions: ['txt', 'text']},
          {name: 'All Files', extensions: ['*']}
        ]
      })
      console.log(fname);
    }
//------------------------------------------------------------------------------
//                            InputBox
//------------------------------------------------------------------------------

    handleNewCallsign(callsign)
    {
      var max = moment.utc(0);
      var rv = null;
      this.state.logbook.map((qso)=>{
        var ts= moment(qso["timestamp_"])
        if ((qso["CALL"]==callsign) && (max<ts))
        {
          rv = qso;
          max = ts;
        }
      });
      this.setState({lastSeen:rv});
    }

    handleUserSubmit(qso){
      var translate = {"callsign":"CALL", "his":"RST_SENT", "mine":"RST_RCVD", "date":"QSO_DATE", "time":"TIME_ON", "freq":"FREQ"};
      var out = {};
      for (var key in qso)
        out[key in translate ? translate[key]:key.toUpperCase()] = qso[key];
      this.writeQSO(out);
      return true;
    }

    writeQSO(qso){
      if ((qso.CALL =="") || (qso.RST_SENT==""))
        return false;
      if (qso.QSO_DATE=="")
        qso.QSO_DATE = moment().utc().format("YYYYMMDD");
      if (qso.TIME_ON=="")
        qso.TIME_ON = moment().utc().format("HHmm");
      if (!qso.FREQ)
        qso.FREQ = this.state.freq;
      if (!qso.MODE)
        qso.MODE = this.state.mode;
      if (this.state.antenna)
        qso["MY_ANTENNA"] =  this.state.antenna;
      if (this.state.rig)
        qso["MY_RIG"] = this.state.rig;
      if (this.state.city)
        qso["MY_CITY"] = this.state.city;
      this.logbook.append(qso);
    }

//------------------------------------------------------------------------------
//                               WSJT-X
//------------------------------------------------------------------------------

    heartbeat(msg){
    //      console.log(msg);
    }

    handleWsjtxQso(qso){
      console.log(qso);
      this.writeQSO(qso);
    }

//------------------------------------------------------------------------------
//                           fldigi
//------------------------------------------------------------------------------
    handleFldigiQso(qso){
      console.log(qso);
      this.writeQSO(qso);
    }

//------------------------------------------------------------------------------
//                                  eQSL
//------------------------------------------------------------------------------

    bestMatch(qso, list){
      var rv = null;
      var diff = 9999;
      for (var j in list)
        if ((qso.CALL == list[j].CALL) &&
            (qso.DATE_ON == list[j].DATE_ON))
            {
              var a = qso.timestamp_;
              var b = list[j].timestamp_;
              var duration = Math.abs(moment.duration(a.diff(b)).asMinutes());
              qso["diff_"] = duration;
              if ((duration<=15) || (Math.abs(duration-120)<=15) || (Math.abs(duration-180)<=15))
              {
                if (duration<diff){
                  diff = duration;
                  rv = list[j];
                }
              }
           }
       return rv;
    }

    handleQSLRcvdClick(event){
      var target = event.target;
      this.logbook.getById(event.target.id).then((qso)=>{
        qso["QSL_RCVD"] = target.checked?"N":"Y";
        this.logbook.updateQso(qso)
        })
    }

    handleQSLSentClick(event){
      var target = event.target;
      this.logbook.getById(event.target.id).then((qso)=>{
        qso["QSL_SENT"] = target.checked?"N":"Y";
        this.logbook.updateQso(qso)
        })
    }

    handleShowEQsl(event){
      var id = event.target.id
      const loadView = ({title,imageBase64}) => {
        return (`
          <!DOCTYPE html>
          <html>
            <head>
              <title>${title}</title>
              <meta charset="UTF-8">
            </head>
            <body>
            <img alt="Embedded Image"
                src="data:image/png;base64,${imageBase64}" />
            </body>
          </html>
        `)
      }
      this.logbook.getAttachment(event.target.id, 'eqsl')
      .then((blob) => {
        blobUtil.blobToBase64String(blob).then( (base64String) =>{
          let window = new electron.remote.BrowserWindow({
              width: 528+15, height: 336+32,
              parent: electron.remote.getCurrentWindow(),
              modal: false
            })
          var file = 'data:text/html;charset=UTF-8,' + encodeURIComponent(loadView({
            title: "eQSL "+id,
            imageBase64: base64String
          }));
          window.loadURL(file);
        }).catch( (err) => {
          console.log(err);
        })
      })
    }

    handleSendEQSL(event){
      this.logbook.getById(event.target.id).then((origqso)=>{
        var qso = origqso;
        if (qso["MODE"].startsWith("PSK")) {
          qso["SUBMODE"] = qso["MODE"];
          qso["MODE"] = "PSK"
        }
        qso['QSLMSG'] = this.config.eqsl.msg;
        this.eqsl.sendEQsl(helpers.objectToAdif(qso))
        .then(()=>{
          origqso["QSL_SENT"] = "Y";
          this.logbook.updateQso(origqso)
          })
        .catch((error)=>{})
        })
    }

    handleEqslSync(event){
        this.eqsl.fetchQsls()
        .then((text)=>{
          console.log("merging eqsl");
          var list = this.state.logbook;
          var qsls = helpers.parseAdifFile(text);
          for (var i in qsls){
            this.logbook.calculateQsoFields(qsls[i]);
            var bestMatch = this.bestMatch(qsls[i], list);
            if (bestMatch !== null){
              var needUpdate = false
              // console.log(qso[i],bestMatch);
              // copy qsl fields
              for (var field in qsls[i])
              {
                var f = field.replaceAll("SENT","RCVD")
                if ((field.includes("QSL")) && (bestMatch[f] != qsls[i][field])) {
                  bestMatch[f] = qsls[i][field];
                  needUpdate = true
                }
              }
              if (needUpdate)
                 this.logbook.updateQso(bestMatch);

              // download picture
              if (('_attachments' in bestMatch) && ('eqsl' in bestMatch['_attachments'])){
                // nothing to do
              }
              else {
                this.eqsl.fetchImageAlt(bestMatch["_id"], qsls[i]).then((res)=>{
                  this.eqsl.archive(qsls[i]);
                  this.logbook.attachImageFromUrl(res.id,"eqsl",res.url);
                })
              }

            }
            else {
              console.log("new ",qsls[i]);
            }
          }

        });
    }

//------------------------------------------------------------------------------
// render method is most important
// render method returns JSX template
//------------------------------------------------------------------------------
    render() {
        return (
          <div id="grid">
            <div id="toolbarLeftArea" className="toolbar-button">
               <InputToolbar freq = {this.state.freq}
                        mode={this.state.mode}
                        rigs={this.config.rig}
                        rig={this.state.rig}
                        antennas={this.config.antenna}
                        antenna={this.state.antenna}

                        onFreqChanged={this.handleToolbarChanged.bind(this)}
                        onModeChanged={this.handleToolbarChanged.bind(this)}
                        onSomethingChanged={this.handleToolbarChanged.bind(this)}
                        />
            </div>
            <div id="toolbarRightArea" className="toolbar-button">
               <QsoListToolbar
                        onSearch={this.handleSearch.bind(this)}
                        onEqslSync={this.handleEqslSync.bind(this)}
                        onExportAdif={this.handleExportAdif.bind(this)}
                        onExportPDF={this.handleExportPDF.bind(this)}
                        onImportAdif={this.handleImportAdif.bind(this)}
                        />
            </div>
            <div id="inputArea">
               <Tabs>
                 <Tab name="Input">
                 <InputBox onSubmit={this.handleUserSubmit.bind(this)}
                           onNewCallsign={this.handleNewCallsign.bind(this)}
                           lastSeen={this.state.lastSeen}
                           />
                 </Tab>
                 <Tab name="Stats">
                 <StatsBox logbook={this.state.logbook}/>
                 </Tab>
                 <Tab name="Settings">
                 <SettingsBox settings={this.config}/>
                 </Tab>
                 </Tabs>
            </div>
            <div id="logArea">
              <LogBox logbook={this.state.logbook}
                      isLoaded={this.state.isLoaded}
                      filterText={this.state.searchText}
                      onRcvdClick = {this.handleQSLRcvdClick.bind(this)}
                      onSentClick = {this.handleQSLSentClick.bind(this)}
                      onSendEQsl = {this.handleSendEQSL.bind(this)}
                      onShowEQsl = {this.handleShowEQsl.bind(this)}
                      />
            </div>
            <div id="footerArea">
               <FooterBox count={this.state.logbook.length} stat={this.state.pluginStat}/>
            </div>
          </div>
        );
    }
}

// Render to ID content in the DOM
ReactDOM.render( < App / > ,
    document.getElementById('content')
);
