import React, { Component } from 'react';
var moment = require('moment');
import { DropdownButton, DropdownButtonItem} from './ui'

//------------------------------------------------------------------------------
//                             L E F T
//------------------------------------------------------------------------------

class InputToolbar extends Component {
  constructor(props){
    super(props)
    this.bands = {
      "1.8":"160m",
      "3.5":"80m",
      "7":"40m",
      "10":"30m",
      "14":"20m",
      "18":"17m",
      "21":"15m",
      "25":"12m",
      "28":"10m",
      "50":"6m",
      "144":"2m",
      "220":"1.25m",
      "432":"70cm",
      "902":"35cm",
      "1300":"23cm",
      "2300":"13cm",
      "3300":"9cm",
      "5660":"6cm",
      "10000":"3cm",
      "24000":"1.25cm",
      "47000":"6mm",
      "75000":"4mm",
      "120000":"2.5mm",
      "142000":"2mm",
      "241000":"1mm"
    }
    this.freqs = [];
    for (var f in this.bands)
          this.freqs.push(<option value={f} key={f} >{f>1000?(f/1000)+" GHz":f+" MHz"} / {this.bands[f]} </option>);
    this.freqs.sort((a,b) => { return parseInt(a.key) - parseInt(b.key)});
  }

  render() {
    var rigs = this.props.rigs.map((rig,i )=>{return <option key={i} value={rig.replace(" ","_")}>{rig}</option>});
    rigs.unshift(<option key="999" value="">None</option>);
    var antennas = this.props.antennas.map((ant,i )=>{return <option key={i} value={ant.replace(" ","_")}>{ant}</option>});
    antennas.unshift(<option key="999" value="">None</option>);
    return (
      <div>
      <select id="freq" value={this.props.freq} onChange={this.props.onFreqChanged}>{this.freqs}</select>
      {" "}
      <select id="mode" value={this.props.mode} onChange={this.props.onModeChanged}>
        <option value="SSB">SSB</option>
        <option value="CW">CW</option>
        <option value="RTTY">RTTY</option>
        <option value="AMTOR">AMTOR</option>
        <option value="PKT">PKT</option>
        <option value="AM">AM</option>
        <option value="FM">FM</option>
        <option value="SSTV">SSTV</option>
        <option value="ATV">ATV</option>
        <option value="PACTOR">PACTOR</option>
        <option value="CLOVER">CLOVER</option>
        <option value="PSK31">PSK31</option>
        <option value="PSK63">PSK63</option>
        <option value="JT65">JT65</option>
        <option value="JT9">JT9</option>
        <option value="FT8">FT8</option>
      </select>
      <br/>
      <br/>
      <select id="rig" value={this.props.rig} onChange={this.props.onSomethingChanged}>
      {rigs}
      </select>
      {" "}
      <select id="antenna" value={this.props.antenna} onChange={this.props.onSomethingChanged}>
      {antennas}
      </select>
      {" "}
      <input type="text" id="city" name="city" size="12" placeholder="location..." value={this.props.city} onChange={this.props.onSomethingChanged} />
      <br/>
      <br/>
      </div>
    );
  }
}

//------------------------------------------------------------------------------
//                              R I G H T
//------------------------------------------------------------------------------

class QsoListToolbar extends Component {
  constructor(props){
    super(props)
    this.intervalId = setInterval(this.timerTick.bind(this), 1000);
    this.state = {datetime:""}
  }
  componentWillUnmount(){
    clearInterval(this.intervalId);
  }
  timerTick(){
    this.locale = moment().locale("en_gb");         // en
    this.setState({datetime:this.locale.utc().format("l LT")});
  }
  searchChanged(event){
    this.props.onSearch(event.target.value);
  }
  render() {
    return (
      <div>
      <font face="verdana" >
      {this.state.datetime}
      </font>
      <br/>
      <br/>
      <button onClick={this.props.onEqslSync}><i className="fa fa-refresh" aria-hidden="true"></i> eQSL</button>
      {" | "}
      <DropdownButton title="Export" icon="fa-download">
        <DropdownButtonItem onClick={this.props.onExportPDF}>PDF</DropdownButtonItem>
        <DropdownButtonItem onClick={this.props.onExportAdif}>Adif</DropdownButtonItem>
      </DropdownButton>
      {" | "}
      <DropdownButton title="Import" icon="fa-upload">
        <DropdownButtonItem onClick={this.props.onImportAdif}>Adif</DropdownButtonItem>
      </DropdownButton>
      {" | "}
      <input type="text" id="logfilter" className="" placeholder="Search" onChange={this.searchChanged.bind(this)}/>
      <button onClick={(event)=>{document.getElementById('logfilter').value = ""; this.searchChanged(event)}}><i className="fa fa-times"></i></button>
      </div>

    );
  }
}

export {InputToolbar, QsoListToolbar}
